%% Estimation of the lower part of the distribution using a Singh-Maddala functional form
% 
% Steps of the program:
%   * 1. Computes the density and its derivative at p1
%   * 2. Defines the CDF, the PDF, its derivative and the quantile function of the Singh-Maddala distribution
%   * 3. Converts these symbolic functions to Matlab functions
%   * 4. Minimizes fmin and adjusts p0
%   * 5. Estimates Q_low with optimal parameters
%
function [Q_low] = lower_part2(q,q_pr,q_sec,p1,p0_max,mu,P_low,p0)

%% 1. Computes the density and its derivative at p1
f1 = 1/q_pr;
f1_pr = -(f1)^3*q_sec;

%% 2. Defines the CDF, the PDF, its derivative and the quantile function of the Singh-Maddala distribution
syms p y c k s z

F_sym = z+1-(1+(s*y).^c).^(-k); % CDF
f_sym = diff(F_sym,y); % PDF
f_sym_pr = diff(f_sym,y); % derivative of the PDF
Q_sym = (1/s)*((1-p).^(-1/k)-1).^(1/c); % quantile function

%% 3. Converts these symbolic functions to Matlab functions

F = subs(F_sym,y,q); % evaluates the CDF at q=Q(p1)
f = subs(f_sym,y,q); % evaluates the PDF at q
f_pr = subs(f_sym_pr,y,q); % evaluates the derivative of the PDF at q

F_func = matlabFunction(F); % converts F
f_func = matlabFunction(f); % converts f
f_pr_func = matlabFunction(f_pr); % converts f_pr
Q_func = matlabFunction(Q_sym); % converts Q

%% 4. Defines the functional to be minimized
fmin = @(c,k,s,z)(1000*(F_func(c,k,s,z)-p1).^2+20*(f_func(c,k,s)-f1).^2+(f_pr_func(c,k,s)-f1_pr).^2);
% fmin is minimal for parameters c, k and s such that F=p1, f=f1 and
% f_pr=f1_pr

%% 5. Minimizes fmin and adjusts p0
if isnan(p0)
N=20; % number of nodes in the mesh
opt = ones(N,5); % matrix of parameters corresponding to each of the N values of p0

iter = 0; % number of iterations
p0_L = 0; % lower value of the search interval for p0
p0_U = p0_max; % upper value
c_temp=1; % criterion for optimality

while (c_temp>0.01)&&(iter<10)
iter = iter+1;
%    parfor i=1:N
    for i=1:N
        p0 = (1-(i-1)/(N-1))*p0_L+((i-1)/(N-1))*p0_U;
        
        options = optimoptions('fmincon','Display','off');
        param = fmincon(@(x)fmin(x(1),x(2),x(3),p0),[2,1,1],[],[],[],[],[0,0,0],[20,20,20],[],options);
    
        p1_temp = F_func(param(1),param(2),param(3),p0);
        f1_temp = f_func(param(1),param(2),param(3));
        f1_pr_temp = f_pr_func(param(1),param(2),param(3));
        m_temp = (1/F_func(param(1),param(2),param(3),p0))*integral(@(p)(Q_func(param(1),param(2),p,param(3))),0,F_func(param(1),param(2),param(3),p0)-p0);
        
        opt(i,:)=[abs(p1_temp-p1)/p1+abs(f1_temp-f1)/f1+abs((f1_pr_temp-f1_pr)/f1_pr)+100*abs(m_temp-mu)/mu param(1) param(2) param(3) p0];
    end
opt
    [c_temp,I] = min(opt(:,1));
    
    if I>1
        p0_L = opt(I-1,5);
    end
    if I<N
        p0_U = opt(I+1,5);
    end
end

%% 6. Estimates Q_low with optimal parameters
[~,I] = min(opt(:,1));
opt(:,1)

Q_low = zeros(size(P_low,1),1);
Q_low((F_func(opt(I,2),opt(I,3),opt(I,4),opt(I,5))/p1)*P_low>opt(I,5)) = Q_func(opt(I,2),opt(I,3),(F_func(opt(I,2),opt(I,3),opt(I,4),opt(I,5))/p1)*P_low((F_func(opt(I,2),opt(I,3),opt(I,4),opt(I,5))/p1)*P_low>opt(I,5))-opt(I,5),opt(I,4));

%% 7. If p0 is provided
else
    options = optimoptions('fmincon','Display','off');
    param = fmincon(@(x)fmin(x(1),x(2),x(3),p0),[2,1,1],[],[],[],[],[0,0,0],[20,20,20],[],options);
    
    Q_low = zeros(size(P_low,1),1);
    Q_low((F_func(param(1),param(2),param(3),p0)/p1)*P_low>p0) = Q_func(param(1),param(2),(F_func(param(1),param(2),param(3),p0)/p1)*P_low((F_func(param(1),param(2),param(3),p0)/p1)*P_low>p0)-p0,param(3));
end    

disp('Lower part estimated')
