%% Estimation of the generalized Pareto curve b(p) using income tax data
%
% Method: PCHIP interpolation
%
% Inputs:
%   * pp: table of percentiles (in [0,1]) corresponding to the thresholds in the tax
%   tabulation
%   * bb: table of (inverted) Pareto coefficients corresponding to the
%   thresholds in the tax tabulation
%   * P: table of points p (in [0,1]) where b(p) has to be calculated
%
% Output:
%   * B: table of approximative values taken by b at points of the input
%   table P, that is, B=b(P)
%
% Steps of the program:
%   1. Preliminaries
%   2. Interpolation of the Pareto curve above the lower threshold using
%   PCHIP method
%
%% 1. Preliminaries
% Defines function inputs and outputs.

function [pp_mid,bb_out] = pc2(thr,pp,bb)

beta = log(((1-pp(1:(end-1))).*thr(1:(end-1)).*bb(1:(end-1)))./((1-pp(2:end)).*thr(2:end).*bb(2:end)));

w_tot = sum(thr(2:end).^2);
w = thr(2:end).^2/w_tot;
1/w_tot;

%% 2. Interpolation
mid = (pp(1:(end-1))+pp(2:end))/2;
%pp_mid = [pp; mid];
pp_mid = [reshape([pp(1:end-1) mid],[],1); pp(end)];

f = @(p,bm)(interp1(pp_mid,[reshape([bb(1:end-1) bm],[],1); bb(end)],p,'pchip','extrap'));

f_beta = @(p,bm)(quadgk(@(q)(1./((1-q).*f(q,bm))),pp(1),p,'MaxIntervalCount',10000));
bm_temp = interp1(pp,bb,mid,'pchip','extrap');

%f_beta(bb(2),bm_temp)
%fmin = @(bm)(sum((f_beta(bb(2:end),bm)-beta).^2));
    syms bm

fmin = @(bm)((f_beta(pp(2),bm)-beta(1)).^2);
for i=3:length(bb)
    fmin = @(bm)(fmin(bm)+(f_beta(pp(i),bm)-f_beta(pp(i-1),bm)-beta(i-1)).^2);
end
fmin(bm_temp)

max_bm = (bb(1:(end-1))+bb(2:end))/2;

%options = optimoptions('fmincon','Display','off','MaxIter',20000,'TolFun',1e-6);
options = optimoptions('fmincon','MaxIter',20000,'TolFun',1e-6);

bb_mid = fmincon(@(bm)fmin(bm),bm_temp,[],[],[],[],ones(length(bb)-1,1),[],[],options);

fmin(bb_mid)


%exp(f_beta(pp(end),bb_mid)-beta(end))
for i=1:length(beta)
    i
exp(-f_beta(pp(i+1),bm_temp)+sum(beta(1:i)))
exp(-f_beta(pp(i+1),bb_mid)+sum(beta(1:i)))
end


bb_out = [reshape([bb(1:end-1) bb_mid],[],1); bb(end)];

pp_draw = linspace(pp(1)-0.05,1,10000);
bb_draw = interp1(pp_mid,bb_out,pp_draw,'pchip','extrap');
plot(pp_draw,bb_draw)
hold on
scatter(pp,bb)

disp('Pareto curve estimated.')