%% Estimation of the generalized Pareto curve b(p) using income tax data
%
% Method: PCHIP interpolation
%
% Inputs:
%   * pp: table of percentiles (in [0,1]) corresponding to the thresholds in the tax
%   tabulation
%   * bb: table of (inverted) Pareto coefficients corresponding to the
%   thresholds in the tax tabulation
%   * P: table of points p (in [0,1]) where b(p) has to be calculated
%
% Output:
%   * B: table of approximative values taken by b at points of the input
%   table P, that is, B=b(P)
%
% Steps of the program:
%   1. Preliminaries
%   2. Interpolation of the Pareto curve above the lower threshold using
%   PCHIP method
%
%% 1. Preliminaries
% Defines function inputs and outputs.

function [pp_mid,bb_out] = pc3(thr,pp,bb,beta)

%% 2. Interpolation
omega = length(pp);
n = 3;
pp_opt = zeros(n*omega+1,1);
for i=1:(omega-1)
    pp_opt(((n+1)*i+1):((n+1)*(i+1)+1)) = linspace(pp1(i),pp1(i+1),n+2);
end

%mid = (pp(1:(end-1))+pp(2:end))/2;
%pp_mid = [pp; mid];
%pp_mid = [reshape([pp(1:end-1) mid],[],1); pp(end)];

f = @(p,bb_opt)(interp1(pp_opt,[reshape([bb(1:end-1) bb_opt],[],1); bb(end)],p,'pchip','extrap'));

f_beta = @(p,bb_opt)(quadgk(@(q)(1./((1-q).*f(q,bb_opt))),pp(1),p,'MaxIntervalCount',10000));
bb_opt_temp = interp1(pp,bb,pp_opt);
bb_opt_temp = reshape(bb_opt_temp(1:end-1),omega,n+1);


%f_beta(bb(2),bm_temp)
%fmin = @(bm)(sum((f_beta(bb(2:end),bm)-beta).^2));

fmin = @(bm)((f_beta(pp(2),bm)-beta(1)).^2);
for i=3:length(bb)
    fmin = @(bm)(fmin(bm)+(f_beta(pp(i),bm)-f_beta(pp(i-1),bm)-beta(i-1)).^2);
end
fmin(bb_opt_temp)

max_bm = (bb(1:(end-1))+bb(2:end))/2;

options = optimoptions('fmincon','Display','off','MaxIter',2000,'TolCon',1e-10,'TolFun',1e-10);
bb_mid = fmincon(@(bm)fmin(bm),bm_temp,[],[],[],[],ones(length(bb)-1,1),[],[],options);

fmin(bb_mid)

%B = interp1(pp_mid,[reshape([bb(1:end-1) bb_mid],[],1); bb(end)],P,'pchip','extrap');

%B(P<pp(1,1)-0.05) = 1;

%exp(f_beta(pp(end),bb_mid)-beta(end))
for i=1:length(beta)
exp(-f_beta(pp(i+1),bm_temp)+sum(beta(1:i)))
exp(-f_beta(pp(i+1),bb_mid)+sum(beta(1:i)))
end



bb_out = [reshape([bb(1:end-1) bb_mid],[],1); bb(end)];

pp_draw = linspace(0,1,10000);
bb_draw = interp1(pp_mid,bb_out,pp_draw,'pchip','extrap');
plot(pp_draw,bb_draw)
hold on
scatter(pp,bb)

disp('Pareto curve estimated.')