%% Estimation of the generalized Pareto curve b(p) using income tax data
%
% Method: PCHIP interpolation
%
% Inputs:
%   * pp: table of percentiles (in [0,1]) corresponding to the thresholds in the tax
%   tabulation; the first point is an additional point to set the average
%   of the lower part of the distribution (below the first raw threshold)
%   * bb: table of (inverted) Pareto coefficients corresponding to the
%   thresholds in the tax tabulation; the first point is an additional point to set the average
%   of the lower part of the distribution
%   * PP: table of points p (in [0,1]) where b(p) has to be calculated
%
% Output:
%   * BB: table of approximative values taken by b at points of the input
%   table PP, that is, BB=b(PP)
%
% Steps of the program:
%   1. Preliminaries
%   2. Interpolation of the Pareto curve above the lower threshold using
%   PCHIP method
%

%% 1. Function pc_fin

function [BB] = pc_fin(thr,pp,bb,PP)

%% 1.1. Preliminaries
% Defines function inputs and outputs.

% output vector BB
BB = ones(size(PP));

% vector beta
beta = log(((1-pp(1:(end-1))).*thr(1:(end-1)).*bb(1:(end-1)))./((1-pp(2:end)).*thr(2:end).*bb(2:end)));

% final point for the PC interpolation
pf = 1-(1-pp(end))/2;
bf = interp1([pp(end-1); pp(end)],[bb(end-1); bb(end)],pf,'linear','extrap');
bf = max(bf,1);
pp = [pp(:); pf];
bb = [bb(:); bf];

% length of vectors pp and bb
T = length(pp);

% epsilon determining the precision of the approximation of the beta(i)
eps = 1e-14;

% time limit for while loops
timelimit = 20;

%% 1.2. Approximation of the upper part (above the first threshold)

% vectors of all points that have been set
Lp = []; % percentiles
Lb = []; % corresponding b

% browse through the raw brackets to set the intermediary points so that
% the successive integrals of 1/((1-p)b(p)) are equals to the betas

% pp(2) is the first "raw" threshold, pp(T-1) is the last one

for i=2:T-2
    i
    
    % points of interest in the loop: p2 and p3 delimit the bracket where
    % the mobile points lie, p1 and p4 are the surronding percentiles
    p1 = pp(i-1);
    p2 = pp(i);
    p3 = pp(i+1);
    p4 = pp(i+2);
    
    % corresponding Pareto coefficients
    b1 = bb(i-1);
    b2 = bb(i);
    b3 = bb(i+1);
    b4 = bb(i+2);
    
    % call to the function f_beta that will compute the intermediary points
    [BB,qq,bbq] = f_beta(p1,p2,p3,p4,b1,b2,b3,b4,Lp,Lb,beta(i),PP,BB,eps);

    Lp = [Lp(:); p2; qq(:)];
    Lb = [Lb(:); b2; bbq(:)];
    
end

Lp = [Lp(:); pp(end-1)];
Lb = [Lb(:); bb(end-1)];

%% 1.3. Approximation of the upper part (above the last threshold)

% eta = min(1e-4,(pp(end-1)-pp(end-2))/4);
% b0 = interp1(Lp,Lb,pp(end-1)-eta,'pchip','extrap');
% 
% 
% % extend the approximation linearly above the last threshold
% BB(PP>=pp(end-1)) = interp1([pp(end-1)-eta; pp(end-1)],[b0; bb(end-1)],PP(PP>=pp(end-1)),'linear','extrap');
% BB = max(BB,ones(size(BB))); % ensure that the Pareto coefficients are above 1

BB(PP>=pp(end-1)) = interp1([Lp(:); pf],[Lb(:); bf],PP(PP>=pp(end-1)),'pchip','extrap');
BB = max(BB,ones(size(BB))); % ensure that the Pareto coefficients are above 1

%% 1.4. Approximation of the lower part

% beta btween pp(1) and pp(2)
beta1 = log(((1-pp(1)).*thr(1).*bb(1))./((1-pp(2)).*thr(2).*bb(2)));

% percentiles delimiting the lower part
p1 = pp(1);
p2 = pp(2);

% mobile points in the lower part
q1 = 0.999*p1+0.001*p2;
q2 = 0.6*p1+0.4*p2;
% these two mobile points will be adjusted vertically

% vectors of percentiles in the whole range [0,1]
pp_low = [p1; q1; q2; Lp(:)];

% value of the Pareto curve at point p given its values bq1 and bq2 at
% mobile points m1 and m2
b_low = @(p,bq1,bq2)(interp1(pp_low,[bb(1); bq1; bq2; Lb(:)],p,'pchip','extrap'));

% function giving the value of the integral between points p1 and p2 as a
% function of the values bq1 and bq2 of the Pareto curve at q1 and q2
int_low = @(bq1,bq2)(quadgk(@(q)(1./((1-q).*b_low(q,bq1,bq2))),pp(1),pp(2),'MaxIntervalCount',10000));

% initial guess of bq1 and bq2
bq1_init = interp1(pp(2:3),bb(2:3),q1,'linear','extrap');
bq2_init = interp1(pp(2:3),bb(2:3),q2,'linear','extrap');

temp_low = int_low(bq1_init,bq2_init);

if temp_low>beta1 %1st case: if the initial guess of the integral is too high, meaning that the bq1_init and bq2_init are too low
    
    bq1_l = bq1_init;
    bq1_h0 = interp1(pp(1:2),bb(1:2),q1,'linear','extrap');
    bq1_h = bq1_h0;
    
    bq2_l = bq2_init;
    bq2_h0 = interp1(pp(1:2),bb(1:2),q2,'linear','extrap');
    bq2_h = bq2_h0;
    
    while int_low(bq1_h,bq2_h)>beta1
        bq1_h = bq1_h+(bq1_h0-bq1_l)/4;
        bq2_h = bq2_h+(bq2_h0-bq2_l)/4;
        int_low(bq1_h,bq2_h)-beta1
        pause
    end
        
    % adjust bq1 and bq2 while the integral temp_low is not close enough to
    % beta1
    t = cputime;
    while abs(temp_low-beta1)>eps
        
        bq1_temp = (bq1_h+bq1_l)/2;
        bq2_temp = (bq2_h+bq2_l)/2;
        
        if int_low(bq1_temp,bq2_temp)>beta1
            bq1_l = bq1_temp;
            bq2_l = bq2_temp;
        else
            bq1_h = bq1_temp;
            bq2_h = bq2_temp;
        end
        
        temp_low = int_low(bq1_temp,bq2_temp);
        if cputime-t > timelimit
            break
        end
    end
    
else %2nd case: if the initial guess of the integral is too low, meaning that the bq1_init and bq2_init are too high
    
    bq1_l = bq1_init;
    bq1_h = bq1_init;
    
    bq2_l = bq2_init;
    bq2_h = bq2_init;
    
    while int_low(bq1_l,bq2_l)<=beta1
        bq1_l = bq1_l-1;
        bq2_l = bq2_l-1;
    end
        
    % adjust bq1 and bq2 while the integral temp_low is not close enough to
    % beta1
    t = cputime;
    while abs(temp_low-beta1)>eps
        
        bq1_temp = (bq1_h+bq1_l)/2;
        bq2_temp = (bq2_h+bq2_l)/2;
        
        if int_low(bq1_temp,bq2_temp)>beta1
            bq1_l = bq1_temp;
            bq2_l = bq2_temp;
        else
            bq1_h = bq1_temp;
            bq2_h = bq2_temp;
        end
        
        temp_low = int_low(bq1_temp,bq2_temp);
        
        if cputime-t > timelimit
            break
        end
    end
    
end

BB(PP<=p2) = b_low(PP(PP<=p2),bq1_temp,bq2_temp);

disp('Pareto curve estimated')

end

function [BB,qq,bbq] = f_beta(p1,p2,p3,p4,b1,b2,b3,b4,Lp,Lb,beta0,PP,BB,eps)

% minimal separation between mobile percentiles and bracket bounds
sep = 1/5;

% Pareto curve interpolation
b_up = @(p,p2,q1,q2,p3,b2,bq1,bq2,b3,Lp,Lb)(interp1([Lp(:); p2;q1;q2;p3],[Lb(:); b2;bq1;bq2;b3],p,'pchip','extrap'));

% beta-integral between p2 and p3
int_up = @(p2,q1,q2,p3,b2,bq1,bq2,b3,Lp,Lb)(quadgk(@(q)(1./((1-q).*b_up(q,p2,q1,q2,p3,b2,bq1,bq2,b3,Lp,Lb))),p2,p3,'MaxIntervalCount',10000));

% distance between 2 points
distance = @(x1,y1,x2,y2)(((x2-x1).^2+(y2-y1).^2).^0.5);

% time limit for while loops
timelimit = 20;

q1_init = (2/3)*p2+(1/3)*p3;
q2_init = (1/3)*p2+(2/3)*p3;

bq1_init = (2/3)*b2+(1/3)*b3;
bq2_init = (1/3)*b2+(2/3)*b3;

temp = int_up(p2,q1_init,q2_init,p3,b2,bq1_init,bq2_init,b3,Lp,Lb);


if temp>beta0 %1st case: if the initial guess of the integral is too high, meaning that the bq1_init and bq2_init are too low
    
    q1_l = q1_init;
    q1_h = q1_init;
    
    bq1_l = bq1_init;
    bq1_h = bq1_init;
    
    q2_l = q2_init;
    q2_h = q2_init;
    
    bq2_l = bq2_init;
    bq2_h = bq2_init;
    
    % slopes of the successive segments
    d1 = (b2-b1)/(p2-p1);
    d2 = (b3-b2)/(p3-p2);
    d3 = (b4-b3)/(p4-p3);
    
    if (d1>d2)||(d2>d3)
        
        %q1, left
         q1_hl = fzero(@(p)(d1*(p-p1)+b1-((-1/d2)*(p-q1_init)+bq1_init)),0.5);
        bq1_hl = d1*(q1_hl-p1)+b1;
        
        %q1, right
         q1_hr = fzero(@(p)(d3*(p-p3)+b3-((-1/d2)*(p-q1_init)+bq1_init)),0.5);
        bq1_hr = d3*(q1_hr-p3)+b3;
        
        %q2, left
         q2_hl = fzero(@(p)(d1*(p-p1)+b1-((-1/d2)*(p-q2_init)+bq2_init)),0.5);
        bq2_hl = d1*(q2_hl-p1)+b1;
        
        %q2, right
         q2_hr = fzero(@(p)(d3*(p-p3)+b3-((-1/d2)*(p-q2_init)+bq2_init)),0.5);
        bq2_hr = d3*(q2_hr-p3)+b3;
        
        if (d1>d2)&&(d2>d3)
            if distance(q1_init,bq1_init,q1_hl,bq1_hl)<distance(q1_init,bq1_init,q1_hr,bq1_hr)
                q1_h = q1_hl;
                bq1_h = bq1_hl;
            else
                q1_h = q1_hr;
                bq1_h = bq1_hr;
            end
            if distance(q2_init,bq2_init,q2_hl,bq2_hl)<distance(q2_init,bq2_init,q2_hr,bq2_hr)
                q2_h = q2_hl;
                bq2_h = bq2_hl;
            else
                q2_h = q2_hr;
                bq2_h = bq2_hr;
            end
        elseif d1>d2
            q1_h = q1_hl;
            bq1_h = bq1_hl;
            q2_h = q2_hl;
            bq2_h = bq2_hl;
        else
            q1_h = q1_hr;
            bq1_h = bq1_hr;
            q2_h = q2_hr;
            bq2_h = bq2_hr;
        end
        
        u = -d2/sqrt(1+d2^2);
        v = 1/sqrt(1+d2^2);
        
        vert = 0;
        if (q1_h-p2<=sep*(p3-p2))||(p3-q2_h<=sep*(p3-p2))
            vert=1;
        end
        
        while (int_up(p2,q1_h,q2_h,p3,b2,bq1_h,bq2_h,b3,Lp,Lb)>beta0)&&(q1_h-p2>sep*(p3-p2))&&(p3-q2_h>sep*(p3-p2))
            
             q1_h = q1_h+u;
             q2_h = q2_h+u;
            
            bq1_h = bq1_h+v;
            bq2_h = bq2_h+v;
                        
            if (q1_h-p2<=sep*(p3-p2))||(p3-q2_h<=sep*(p3-p2))
                vert=1;
            end
        end
        
        if vert == 1
            q1_h = (1-sep)*p2+sep*p3;
            q2_h = sep*p2+(1-sep)*p3;
                        
            while int_up(p2,q1_h,q2_h,p3,b2,bq1_h,bq2_h,b3,Lp,Lb)>beta0
                bq1_h = bq1_h+1;
                bq2_h = bq2_h+1;
            end
        end
    else
        
        % (u,v) is a unitary normal vector of the segment of interest
        u = -d2/sqrt(1+d2^2);
        v = 1/sqrt(1+d2^2);
        
        vert = 0;
        if (q1_h-p2<=sep*(p3-p2))||(p3-q2_h<=sep*(p3-p2))
            vert=1;
        end
        
        while (int_up(p2,q1_h,q2_h,p3,b2,bq1_h,bq2_h,b3,Lp,Lb)>beta0)&&(q1_h-p2>sep*(p3-p2))&&(p3-q2_h>sep*(p3-p2))
             q1_h = q1_h+u;
             q2_h = q2_h+u;
            
            bq1_h = bq1_h+v;
            bq2_h = bq2_h+v;
                        
            if (q1_h-p2<=sep*(p3-p2))||(p3-q2_h<=sep*(p3-p2))
                vert=1;
            end
        end
        
        if vert == 1
            q1_h = (1-sep)*p2+sep*p3;
            q2_h = sep*p2+(1-sep)*p3;
            
            while int_up(p2,q1_h,q2_h,p3,b2,bq1_h,bq2_h,b3,Lp,Lb)>beta0
                bq1_h = bq1_h+1;
                bq2_h = bq2_h+1;
            end
        end
    end
    
    % adjust bq1 and bq2 while the integral temp_low is not close enough to
    % beta1
    t = cputime;
    while abs(temp-beta0)>eps
        
         q1_temp = (q1_h+q1_l)/2;
         q2_temp = (q2_h+q2_l)/2;
        
        bq1_temp = (bq1_h+bq1_l)/2;
        bq2_temp = (bq2_h+bq2_l)/2;
        
        if int_up(p2,q1_temp,q2_temp,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb)>beta0
             q1_l = q1_temp;
             q2_l = q2_temp;
            bq1_l = bq1_temp;
            bq2_l = bq2_temp;
        else
             q1_h = q1_temp;
             q2_h = q2_temp;
            bq1_h = bq1_temp;
            bq2_h = bq2_temp;
        end
        
        temp = int_up(p2,q1_temp,q2_temp,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
        
        if cputime-t > timelimit
            break
        end
    end
    
    BB((PP>=p2)&(PP<=p3)) = b_up(PP((PP>=p2)&(PP<=p3)),p2,q1_temp,q2_temp,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
    qq = [q1_temp; q2_temp];
    bbq = [bq1_temp; bq2_temp];
    
else %2nd case: if the initial guess of the integral is too low, meaning that the bq1_init and bq2_init are too high
    
    q1_l = q1_init;
    q1_h = q1_init;
    
    bq1_l = bq1_init;
    bq1_h = bq1_init;
    
    q2_l = q2_init;
    q2_h = q2_init;
    
    bq2_l = bq2_init;
    bq2_h = bq2_init;
    
    % slopes of the successive segments
    d1 = (b2-b1)/(p2-p1);
    d2 = (b3-b2)/(p3-p2);
    d3 = (b4-b3)/(p4-p3);
    
    if (d1<d2)||(d2<d3)
        
        %q1, left
         q1_ll = fzero(@(p)(d1*(p-p1)+b1-((-1/d2)*(p-q1_init)+bq1_init)),0.5);
        bq1_ll = d1*(q1_ll-p1)+b1;
        
        %q1, right
         q1_lr = fzero(@(p)(d3*(p-p3)+b3-((-1/d2)*(p-q1_init)+bq1_init)),0.5);
        bq1_lr = d3*(q1_lr-p3)+b3;
        
        %q2, left
         q2_ll = fzero(@(p)(d1*(p-p1)+b1-((-1/d2)*(p-q2_init)+bq2_init)),0.5);
        bq2_ll = d1*(q2_ll-p1)+b1;
        
        %q2, right
         q2_lr = fzero(@(p)(d3*(p-p3)+b3-((-1/d2)*(p-q2_init)+bq2_init)),0.5);
        bq2_lr = d3*(q2_lr-p3)+b3;
        
        if (d1<d2)&&(d2<d3)
            if distance(q1_init,bq1_init,q1_ll,bq1_ll)<distance(q1_init,bq1_init,q1_lr,bq1_lr)
                q1_l = q1_ll;
                bq1_l = bq1_ll;
            else
                q1_l = q1_lr;
                bq1_l = bq1_lr;
            end
            if distance(q2_init,bq2_init,q2_ll,bq2_ll)<distance(q2_init,bq2_init,q2_lr,bq2_lr)
                q2_l = q2_ll;
                bq2_l = bq2_ll;
            else
                q2_l = q2_lr;
                bq2_l = bq2_lr;
            end
        elseif d1<d2
            q1_l = q1_ll;
            bq1_l = bq1_ll;
            q2_l = q2_ll;
            bq2_l = bq2_ll;
        else
            q1_l = q1_lr;
            bq1_l = bq1_lr;
            q2_l = q2_lr;
            bq2_l = bq2_lr;
        end
        
        u = -d2/sqrt(1+d2^2);
        v = 1/sqrt(1+d2^2);
        
        vert = 0;
        if (q1_l-p2<=sep*(p3-p2))||(p3-q2_l<=sep*(p3-p2))
        	vert=1;
        end
        
        while (int_up(p2,q1_l,q2_l,p3,b2,bq1_l,bq2_l,b3,Lp,Lb)<=beta0)&&(q1_l-p2>sep*(p3-p2))&&(p3-q2_l>sep*(p3-p2))
             q1_l = q1_l-u;
             q2_l = q2_l-u;
            
            bq1_l = bq1_l-v;
            bq2_l = bq2_l-v;
            
            if (q1_l-p2<=sep*(p3-p2))||(p3-q2_l<=sep*(p3-p2))
                vert=1;
            end
        end
        
        if vert == 1
            q1_l = (1-sep)*p2+sep*p3;
            q2_l = sep*p2+(1-sep)*p3;
            
            while int_up(p2,q1_l,q2_l,p3,b2,bq1_l,bq2_l,b3,Lp,Lb)<=beta0
                bq1_l = bq1_l-1;
                bq2_l = bq2_l-1;
            end
        end
    else
        
        % (u,v) is a unitary normal vector of the segment of interest
        u = -d2/sqrt(1+d2^2);
        v = 1/sqrt(1+d2^2);
        
        vert = 0;
        if (q1_l-p2<=sep*(p3-p2))||(p3-q2_l<=sep*(p3-p2))
        	vert=1;
        end
        
        while (int_up(p2,q1_l,q2_l,p3,b2,bq1_l,bq2_l,b3,Lp,Lb)<=beta0)&&(q1_l-p2>sep*(p3-p2))&&(p3-q2_l>sep*(p3-p2))
             q1_l = q1_l-u;
             q2_l = q2_l-u;
            
            bq1_l = bq1_l-v;
            bq2_l = bq2_l-v;
            
            if (q1_l-p2<=sep*(p3-p2))||(p3-q2_l<=sep*(p3-p2))
                vert=1;
            end
        end
        
        if vert == 1
            q1_l = (1-sep)*p2+sep*p3;
            q2_l = sep*p2+(1-sep)*p3;
            
            while int_up(p2,q1_l,q2_l,p3,b2,bq1_l,bq2_l,b3,Lp,Lb)<=beta0
                bq1_l = bq1_l-1;
                bq2_l = bq2_l-1;
            end
        end
    end
    
    
    % adjust bq1 and bq2 while the integral temp is not close enough to
    % beta1
    t=cputime;
    while abs(temp-beta0)>eps
        
         q1_temp = (q1_h+q1_l)/2;
         q2_temp = (q2_h+q2_l)/2;
        
        bq1_temp = (bq1_h+bq1_l)/2;
        bq2_temp = (bq2_h+bq2_l)/2;
                
        if int_up(p2,q1_temp,q2_temp,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb)<beta0
             q1_h = q1_temp;
             q2_h = q2_temp;
            bq1_h = bq1_temp;
            bq2_h = bq2_temp;
        else
             q1_l = q1_temp;
             q2_l = q2_temp;
            bq1_l = bq1_temp;
            bq2_l = bq2_temp;
        end
        
        temp = int_up(p2,q1_temp,q2_temp,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
        
        if cputime-t > timelimit
            break
        end

    end
    
    BB((PP>=p2)&(PP<=p3)) = b_up(PP((PP>=p2)&(PP<=p3)),p2,q1_temp,q2_temp,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
    qq = [q1_temp; q2_temp];
    bbq = [bq1_temp; bq2_temp];
    
end

end
